// -*- C++ -*-
/*!
 * @file  DPSF.cpp
 * @brief Double Plane Segment Finder
 * @date $Date: 2010-04-02 13:39:22 +0900 (金, 02  4月 2010) $
 *
 * $Id: DPSF.cpp 334 2010-04-02 04:39:22Z eus $
 */

#include "DPSF.h"

// Module specification
// <rtc-template block="module_spec">
static const char* dpsf_spec[] =
  {
    "implementation_id", "DPSF",
    "type_name",         "DPSF",
    "description",       "Double Plane Segment Finder",
    "version",           "1.0.0",
    "vendor",            "VenderName",
    "category",          "3D Estimation",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    "exec_cxt.periodic.rate", "1.0",
    // Configuration variables
    "conf.default.ParticleNum", "200",
    "conf.default.ZCenter", "500.0",
    "conf.default.ZRange", "100.0",
    "conf.default.XCenter", "500.0",
    "conf.default.XRange", "100.0",
    "conf.default.RZCenter", "0.0",
    "conf.default.RZRange", "1.57",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
DPSF::DPSF(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_Points3dIn("Points3d", m_Points3d),
    m_PlaneParamsOut("PlaneParams", m_PlaneParams),
    m_TopPlanePoints3dOut("TopPlanePoints3d", m_TopPlanePoints3d),
    m_FrontPlanePoints3dOut("FrontPlanePoints3d", m_FrontPlanePoints3d),
    m_Box3dOut("Box3d", m_Box3d),

    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("Points3d", m_Points3dIn);
  
  // Set OutPort buffer
  registerOutPort("PlaneParams", m_PlaneParamsOut);
  registerOutPort("TopPlanePoints3d", m_TopPlanePoints3dOut);
  registerOutPort("FrontPlanePoints3d", m_FrontPlanePoints3dOut);
  registerOutPort("Box3d", m_Box3dOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  dpsf=NULL;
}

/*!
 * @brief destructor
 */
DPSF::~DPSF()
{
  if(dpsf!=NULL) delete dpsf;
}



RTC::ReturnCode_t DPSF::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("ParticleNum", m_ParticleNum, "200");
  bindParameter("ZCenter", m_ZCenter, "500.0");
  bindParameter("ZRange", m_ZRange, "100.0");
  bindParameter("XCenter", m_XCenter, "500.0");
  bindParameter("XRange", m_XRange, "100.0");
  bindParameter("RZCenter", m_RZCenter, "0.0");
  bindParameter("RZRange", m_RZRange, "1.57");
  
  // </rtc-template>

  if(dpsf!=NULL) delete dpsf;
  dpsf=new HYDPsf(m_ParticleNum,0.1,
                  m_ZCenter,m_ZRange,
                  m_XCenter,m_XRange,
                  m_RZCenter,m_RZRange,
                  //-0.02);
                  -0.0001);

  dpsf->initParticles();

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t DPSF::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DPSF::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DPSF::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DPSF::onActivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DPSF::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t DPSF::onExecute(RTC::UniqueId ec_id)
{
  int len;
  float state[3],param[3];
  int length_top,length_front;

  if(m_Points3dIn.isNew()){
    cout << "in onExecute:" << endl;

    m_Points3dIn.read();
    len=m_Points3d.data.length();
    v_points3d.resize(len);
    int nlen=0;
    float xx,yy,zz;
    for(int i=0;i<len;i+=3){
      xx=m_Points3d.data[i];
      yy=m_Points3d.data[i+1];
      zz=m_Points3d.data[i+2];
      if(xx <= 2000.0 && zz>=0.0){
        v_points3d[nlen]=xx;
        v_points3d[nlen+1]=yy;
        v_points3d[nlen+2]=zz;
        nlen+=3;
      }
    }
    //memcpy(&v_points3d[0],&m_Points3d.data[0],sizeof(float)*len);
    //dpsf->setPoints(&v_points3d[0],len/3);
    dpsf->setPoints(&v_points3d[0],nlen/3);

    cout << "  recieved points: " << len/3 << endl;
    cout << "  survived points: " << nlen/3 << endl;

    // set center automaticaly
    float xave=0.0,zave=0.0;
    float zmax=0.0;
    int cnt=0;
    for(int i=0;i<nlen;i+=3){
      xave+=v_points3d[i];
      zave+=v_points3d[i+2];
      if(zmax<v_points3d[i+2]) zmax=v_points3d[i+2];
      cnt++;
    }
    if(cnt>0){
      xave=xave/cnt;
      //zave=zave/cnt;
      zave=zmax;

      cout << "  xcenter: " << xave << endl;
      cout << "  zcenter: " << zave << endl;

      dpsf->setXCenter(xave);
      dpsf->setXRange(200.0);
      dpsf->setZCenter(zave);
      dpsf->setZRange(200.0);
    }

    dpsf->initParticles();

    for(int i=0;i<3;i++){
      dpsf->resample();
      dpsf->predict();
      dpsf->weight();
      dpsf->update(UPDATE_MAX);
    }

    float state[3];
    dpsf->getState(state);
    float lk=dpsf->likelihood(state);

    cout << "  likelihood: " << lk << endl;

    dpsf->calcOnPlaneFront(30.0);
    dpsf->calcOnPlaneTop(30.0);

    length_front=dpsf->getOnPlaneFrontLength();
    length_top=dpsf->getOnPlaneTopLength();

    cout << "  points(front): " << length_front/3 << endl;
    cout << "  points(top): " << length_top/3 << endl;

    if(length_front>0){
      v_FrontPlanePoints.resize(length_front);
      memcpy(&v_FrontPlanePoints[0],dpsf->getOnPlaneFront(),sizeof(float)*length_front);
      m_FrontPlanePoints3d.data.length(length_front);
      memcpy(&m_FrontPlanePoints3d.data[0],dpsf->getOnPlaneFront(),sizeof(float)*length_front);
      m_FrontPlanePoints3dOut.write();
    }

    if(length_top>0){
      v_TopPlanePoints.resize(length_top);
      memcpy(&v_TopPlanePoints[0],dpsf->getOnPlaneTop(),sizeof(float)*length_top);
      m_TopPlanePoints3d.data.length(length_top);
      memcpy(&m_TopPlanePoints3d.data[0],dpsf->getOnPlaneTop(),sizeof(float)*length_top);
      m_TopPlanePoints3dOut.write();
    }

    m_PlaneParams.data.length(12);
    dpsf->getState(state);
    dpsf->stateToParam(state,param);

    m_PlaneParams.data[0]=param[1];
    m_PlaneParams.data[1]=0.0;
    m_PlaneParams.data[2]=param[0];
    m_PlaneParams.data[3]=0.0;
    m_PlaneParams.data[4]=0.0;
    m_PlaneParams.data[5]=0.0;

    m_PlaneParams.data[6]=param[1];
    m_PlaneParams.data[7]=0.0;
    m_PlaneParams.data[8]=param[0];
    m_PlaneParams.data[9]=0.0;
    m_PlaneParams.data[10]=M_PI*0.5;
    m_PlaneParams.data[11]=param[2];

    m_PlaneParamsOut.write();

    calculateBox(param,v_FrontPlanePoints,v_TopPlanePoints);
    m_Box3dOut.write();
  }

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t DPSF::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DPSF::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DPSF::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DPSF::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DPSF::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

void DPSF::calculateBox(float *param,vector <float> &frontpoints,vector <float> &toppoints){
  m_Box3d.data.length(9);
  // (tx,ty,tz,rx,ry,rz,depth,width,height)
  vector <float> ipoints;

  int lfp=frontpoints.size();
  int ltp=toppoints.size();
  ipoints.resize(lfp+ltp);

  // inverse transformation equation:
  // ix = ( -tx + X) Cos[rz] + Y Sin[rz]
  // iy = Y Cos[rz] + (tx - X) Sin[rz]
  // iz = -tz+z


  float ix,iy,iz;
  float xx,yy,zz;
  float tx=param[1],tz=param[0],rz=param[2];
  float txmx;
  float crz=cos(rz);
  float srz=sin(rz);
  for(int i=0;i<lfp;i+=3){
    xx=frontpoints[i];
    yy=frontpoints[i+1];
    zz=frontpoints[i+2];

    txmx=tx-xx;
    ix=yy*srz-txmx*crz;
    iy=yy*crz+txmx*srz;
    iz=zz-tz;

    ipoints[i]=ix;
    ipoints[i+1]=iy;
    ipoints[i+2]=iz;
  }
  for(int i=0;i<ltp;i+=3){
    xx=toppoints[i];
    yy=toppoints[i+1];
    zz=toppoints[i+2];

    txmx=tx-xx;
    ix=yy*srz-txmx*crz;
    iy=yy*crz+txmx*srz;
    iz=zz-tz;

    ipoints[lfp+i]=ix;
    ipoints[lfp+i+1]=iy;
    ipoints[lfp+i+2]=iz;
  }

  //  size estimation

  // transformation equation:
  // xx = tx + X Cos[rz] - Y Sin[rz]
  // yy = Y Cos[rz] + X Sin[rz]
  // zz = tz + Z

  float xmin,xmax;
  xmax=xmin=ipoints[0];
  float ymin,ymax;
  ymax=ymin=ipoints[1];
  for(int i=3;i<ipoints.size();i+=3){
    xx=ipoints[i];
    yy=ipoints[i+1];

    if(xmin>xx) xmin=xx;
    if(xmax<xx) xmax=xx;
    if(ymin>yy) ymin=yy;
    if(ymax<yy) ymax=yy;
  }

  float xrange,xcenter,yrange,ycenter;
  xrange=xmax-xmin;
  xcenter=(xmax+xmin)*0.5;
  yrange=ymax-ymin;
  ycenter=(ymax+ymin)*0.5;

  xx = tx + xcenter*crz - ycenter*srz;
  yy = ycenter*crz + xcenter*srz;
  zz = tz*0.5;

  m_Box3d.data[0]=xx;
  m_Box3d.data[1]=yy;
  m_Box3d.data[2]=zz;

  m_Box3d.data[3]=0.0;
  m_Box3d.data[4]=0.0;
  m_Box3d.data[5]=rz;

  m_Box3d.data[6]=xrange;
  m_Box3d.data[7]=yrange;
  m_Box3d.data[8]=tz;
}


extern "C"
{
 
  void DPSFInit(RTC::Manager* manager)
  {
    RTC::Properties profile(dpsf_spec);
    manager->registerFactory(profile,
                             RTC::Create<DPSF>,
                             RTC::Delete<DPSF>);
  }
  
};


