// -*- C++ -*-
/*!
 * @file  DPSF.h
 * @brief Double Plane Segment Finder
 * @date  $Date: 2010-04-02 13:39:22 +0900 (金, 02  4月 2010) $
 *
 * $Id: DPSF.h 334 2010-04-02 04:39:22Z eus $
 */

#ifndef DPSF_H
#define DPSF_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

#include "HYDPsf.h"
#include <iostream>
#include <vector>
#include <math.h>

using namespace RTC;
using namespace std;

/*!
 * @class DPSF
 * @brief Double Plane Segment Finder
 *
 */
class DPSF
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  DPSF(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~DPSF();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /*!
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // Configuration variable declaration
  // <rtc-template block="config_declare">
  /*!
   * 
   * - Name:  ParticleNum
   * - DefaultValue: 200
   */
  int m_ParticleNum;
  /*!
   * 
   * - Name:  ZCenter
   * - DefaultValue: 500.0
   */
  float m_ZCenter;
  /*!
   * 
   * - Name:  ZRange
   * - DefaultValue: 100.0
   */
  float m_ZRange;
  /*!
   * 
   * - Name:  XCenter
   * - DefaultValue: 500.0
   */
  float m_XCenter;
  /*!
   * 
   * - Name:  XRange
   * - DefaultValue: 100.0
   */
  float m_XRange;
  /*!
   * 
   * - Name:  RZCenter
   * - DefaultValue: 0.0
   */
  float m_RZCenter;
  /*!
   * 
   * - Name:  RZRange
   * - DefaultValue: 1.57
   */
  float m_RZRange;
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedFloatSeq m_Points3d;
  /*!
   */
  InPort<TimedFloatSeq> m_Points3dIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedFloatSeq m_PlaneParams;
  /*!
   */
  OutPort<TimedFloatSeq> m_PlaneParamsOut;
  TimedFloatSeq m_TopPlanePoints3d;
  /*!
   */
  OutPort<TimedFloatSeq> m_TopPlanePoints3dOut;
  TimedFloatSeq m_FrontPlanePoints3d;
  /*!
   */
  OutPort<TimedFloatSeq> m_FrontPlanePoints3dOut;
  TimedFloatSeq m_Box3d;
  /*!
   */
  OutPort<TimedFloatSeq> m_Box3dOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  int dummy;
  // <rtc-template block="private_attribute">
  HYDPsf *dpsf;
  vector <float> v_points3d;
  vector <float> v_FrontPlanePoints,v_TopPlanePoints;
  // </rtc-template>

  // <rtc-template block="private_operation">
  void calculateBox(float *param,vector <float> &frontpoints,vector <float> &toppoints);
  // </rtc-template>

};


extern "C"
{
  void DPSFInit(RTC::Manager* manager);
};

#endif // DPSF_H
