#include "HYDPsf.h"

HYDPsf::HYDPsf(int np,float rr,
               float zc,float zr,
               float xc,float xr,
               float rzc,float rzr,
               float mn)
  : HYParticleFilter(3,np,rr){
  zcenter=zc;
  zrange=zr;
  xcenter=xc;
  xrange=xr;
  rzcenter=rzc;
  rzrange=rzr;
  mnoise=mn;

  me_k=10.0;
  ransac_r=100.0;

  plane_normal_front=new float[3];
  plane_normal_top=new float[3];
}

HYDPsf::~HYDPsf(){
  delete[] plane_normal_front;
  delete[] plane_normal_top;
}

void HYDPsf::predict(){
  int i;
  float s;
  for(i=0;i<dstate*nparticles;i++){
    s=stateTable[i];
    if(s>0.5){
      stateTable[i]=0.5;
    }else if(s<-0.5){
      stateTable[i]=-0.5;
    }
  }
}

float HYDPsf::likelihood(float *state){
  float val=0.0;

  float dist_f,dist_f2,dist_t,dist_t2,dist2;
  int i,pidx;

  stateToPlane(state);

  dist_t_list.clear();
  dist_f_list.clear();

  for(i=0;i<npoints;i++){
    pidx=i*dstate;
    /*
    dist_f=((points[pidx]*plane_normal_front[0]
             +points[pidx+1]*plane_normal_front[1]
             +points[pidx+2]*plane_normal_front[2])
	  +plane_d_front)/plane_normal_front_norm;
    */
    dist_f=distancePlaneFront(&points[pidx]);
    dist_f2=dist_f*dist_f;

    //dist_t=points[pidx+2]+plane_d_top;
    dist_t=distancePlaneTop(&points[pidx]);
    dist_t2=dist_t*dist_t;

    //if(dist_f2>dist_t2){
    dist2=dist_t2;
    dist_t_list.push_back(dist_t2);
    //}else{
    dist2=dist_f2;
    dist_f_list.push_back(dist_f2);
    //}

    // lms
    //val+=exp(dist2*mnoise);

    // m-estimator
    //val+=exp(dist2/(me_k+dist2)*mnoise);

    // ransac
    //if(dist2<ransac_r) val++;

    // lmeds
    //dist_list[i]=dist2;
  }

  // ransac
  //val*=val;

  // lmeds
  sort(dist_t_list.begin(),dist_t_list.end());
  //dist_t2=dist_t_list[dist_t_list.size()/2];
  dist_t2=dist_t_list[dist_t_list.size()/3];
  sort(dist_f_list.begin(),dist_f_list.end());
  //dist_f2=dist_f_list[dist_f_list.size()/2];
  dist_f2=dist_f_list[dist_f_list.size()/3];

  //val=exp(dist_t2*mnoise)+exp(dist_f2*mnoise);
  val=(exp(dist_t2*mnoise)+1e-5)*(exp(dist_f2*mnoise)+1e-5);

  return val;
}

void HYDPsf::stateToPlane(float *state){
  float zz,xx,rz;

  zz=state[0]*zrange+zcenter;
  xx=state[1]*xrange+xcenter;
  rz=state[2]*rzrange+rzcenter;

  plane_normal_front[0]=cos(rz);
  plane_normal_front[1]=sin(rz);
  plane_normal_front[2]=0.0;

  plane_d_front=-xx*plane_normal_front[0];

  plane_normal_front_norm=sqrt(plane_normal_front[0]*plane_normal_front[0]
                               +plane_normal_front[1]*plane_normal_front[1]
                               +plane_normal_front[2]*plane_normal_front[2]);

  plane_normal_top[0]=0.0;
  plane_normal_top[1]=0.0;
  plane_normal_top[2]=1.0;

  plane_d_top=-zz;

  plane_normal_top_norm=1.0;
}

void HYDPsf::setPoints(float *pts,int num){
  npoints=num;
  points.resize(npoints*3);
  memcpy(&points[0],pts,sizeof(float)*npoints*3);
}

void HYDPsf::stateToParam(float *state,float *param){
  param[0]=state[0]*zrange+zcenter;
  param[1]=state[1]*xrange+xcenter;
  param[2]=state[2]*rzrange+rzcenter;
}

float HYDPsf::distancePlaneFront(float *point){
  return ((point[0]*plane_normal_front[0]
           +point[1]*plane_normal_front[1]
           +point[2]*plane_normal_front[2])
	  +plane_d_front)/plane_normal_front_norm;
}
void HYDPsf::calcOnPlaneFront(float dt){
  float dist;
  int i,j,pidx;

  stateToPlane(state);

  on_plane_front_points.clear();
  
  for(i=0;i<npoints;i++){
    pidx=i*3;
    dist=distancePlaneFront(&points[pidx]);
    if(fabs(dist)<dt)
      for(j=0;j<3;j++) on_plane_front_points.push_back(points[pidx+j]);
  }
}

float HYDPsf::distancePlaneTop(float *point){
  return point[2]+plane_d_top;
}
void HYDPsf::calcOnPlaneTop(float dt){
  float dist;
  int i,j,pidx;

  stateToPlane(state);

  on_plane_top_points.clear();
  
  for(i=0;i<npoints;i++){
    pidx=i*3;
    dist=distancePlaneTop(&points[pidx]);
    if(fabs(dist)<dt)
      for(j=0;j<3;j++) on_plane_top_points.push_back(points[pidx+j]);
  }
}
